#include <SPI.h>
#include "RF24.h"

#define CE 7
#define CS 8
RF24 nRF(CE, CS);
const byte addresses[][6] = {"00001", "00002"};
int codeDisciplie = 11;

#include <Wire.h>
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27, 20, 4);

struct sensor
{
  int id;
  unsigned long long stopTime = 0;
};

struct controlPanel
{
  int gStart;
  int bStart;
  int bReset;
  unsigned long long startTime = 0;
  int countSensors = 2;
  sensor sensors[2];
  int cursorLine;
};

byte y[8] = {B00010, B00100, B10001, B10001, B01111, B00001, B01110, B00000};

struct controlPanel firstControlPanel;
struct controlPanel secondControlPanel;

void setup()
{
  Serial.begin(9600);
  nRF.begin();
  nRF.openWritingPipe(addresses[1]);
  nRF.openReadingPipe(1, addresses[0]);
  nRF.setPALevel(RF24_PA_LOW);
  nRF.startListening();

  lcd.begin();
  lcd.backlight();
  lcd.createChar(0, y);

  writeText();
  setControlPanels(codeDisciplie);
}

void loop()
{
  //start
  if ((digitalRead(firstControlPanel.bStart) || digitalRead(firstControlPanel.gStart)) && firstControlPanel.startTime == 0)
    start(&firstControlPanel);
  else  if (digitalRead(firstControlPanel.bReset) && firstControlPanel.startTime != 0)
    resetSensors(&firstControlPanel);
  if ((digitalRead(secondControlPanel.bStart) || digitalRead(secondControlPanel.gStart)) && secondControlPanel.startTime == 0)
    start(&secondControlPanel);
  else  if (digitalRead(secondControlPanel.bReset) && secondControlPanel.startTime != 0)
    resetSensors(&secondControlPanel);

  if (firstControlPanel.startTime != 0)
    writeTime(&firstControlPanel);
  if (secondControlPanel.startTime != 0)
    writeTime(&secondControlPanel);

  int request;
  if ( nRF.available()) {
    while (nRF.available()) {
      nRF.read( &request, sizeof(request) );
    }
    if (request != 0)
    {
      if (codeDisciplie == 11)
      {
        if (request < 3)
          stopSensor(&firstControlPanel.sensors[request - 1]);
        else
          stopSensor(&secondControlPanel.sensors[request - 2]);
      }
    }
  }
}

void setControlPanels(int code)
{
  if (code == 11)
  {
    //left
    firstControlPanel.gStart = 4;
    firstControlPanel.bStart = 10;
    firstControlPanel.bReset = 9;
    firstControlPanel.cursorLine = 0;
    firstControlPanel.sensors[0].id = 1;
    firstControlPanel.sensors[1].id = 2;

    pinMode(firstControlPanel.gStart, INPUT);
    pinMode(firstControlPanel.bStart, INPUT);
    pinMode(firstControlPanel.bReset, INPUT);
    //right Track
    secondControlPanel.gStart = 3;
    secondControlPanel.bStart = 6;
    secondControlPanel.bReset = 5;
    secondControlPanel.sensors[0].id = 3;
    secondControlPanel.sensors[1].id = 4;
    secondControlPanel.cursorLine = 2;
    pinMode(secondControlPanel.gStart, INPUT);
    pinMode(secondControlPanel.bStart, INPUT);
    pinMode(secondControlPanel.bReset, INPUT);
  }
  else if (code == 22)
  {
    firstControlPanel.gStart = 4;
    firstControlPanel.bStart = 10;
    firstControlPanel.bReset = 9;

    pinMode(firstControlPanel.gStart, INPUT);
    pinMode(firstControlPanel.bStart, INPUT);
    pinMode(firstControlPanel.bReset, INPUT);
  }
}

void start(controlPanel *c)
{
  (*c).startTime = millis();
}

void stopSensor(sensor *s)
{
  (*s).stopTime = millis();
}

void resetSensors(controlPanel *c)
{
  (*c).startTime = 0;
   for (int i = 0; i < (*c).countSensors; i++)
  {
    sendCode((*c).sensors[i].id);
    (*c).sensors[i].stopTime = 0;
  }

}

void sendCode(int code)
{
  nRF.stopListening();
  nRF.write( &code, sizeof(code) );
  nRF.startListening();
}

void  writeText()
{
  lcd.setCursor(0, 0);
  lcd.print("Trat1 Lev");
  lcd.write(byte(0)); 
  lcd.print(":00:00:00");

  lcd.setCursor(0, 1);
  lcd.print("Trat1Prav");
  lcd.write(byte(0)); 
  lcd.print(":");
  lcd.print("00:00:00");

  lcd.setCursor(0, 2);
  lcd.print("Trat2  Lev");
  lcd.write(byte(0)); 
  lcd.print(":00:00:00");

  lcd.setCursor(0, 3);
  lcd.print("Trat2Prav");
  lcd.write(byte(0)); 
  lcd.print(":00:00:00");
}

void writeTime(controlPanel *c)
{

  unsigned long long now = millis() - (*c).startTime;
  String stringTime = createStringTime( millis() - (*c).startTime);
  for (int i = 0; i < (*c).countSensors; i++)
  {
    if ((*c).sensors[(*c).cursorLine + i].stopTime == 0)
    {
      lcd.setCursor(11, (*c).cursorLine + i);
      lcd.print(stringTime);
    }
    else
    {
      stringTime = createStringTime((*c).sensors[(*c).cursorLine + i].stopTime);
      lcd.setCursor(11, (*c).cursorLine + i);
      lcd.print(stringTime);
    }
  }
}

String createStringTime(unsigned long long timeInLong)
{
  unsigned short  m, s, ms;

  String stringTime = "";
  ms = timeInLong % 1000;
  timeInLong /= 1000;
  s = timeInLong % 60;
  timeInLong /= 60;
  m = timeInLong % 60;
  timeInLong /= 60;

  if (valueNumber(m))
    stringTime = "0";
  stringTime = String(stringTime + String(m) + ":");
  if (valueNumber(s))
    stringTime = String(stringTime + "0");
  stringTime = String(stringTime + String(s) + ":");
  if (valueNumber(ms))
    stringTime = String(stringTime + "0");
  stringTime = String(stringTime + String(ms));

  return stringTime;
}

bool valueNumber(int n)
{
  if (n < 10)
    return true;
  return false;
}


