#include <SPI.h>
#include "RF24.h"
#include <stdlib.h>

#define CE 7
#define CS 8
RF24 nRF(CE, CS);
//addresses pipe
const byte addresses[][6] = {"00001", "00002"};
const int countSensors = 4;

struct sensor
{
  int id;
  int pinSensor;
  int pinLed;
  boolean switchOn = false;
};

sensor sensors[countSensors];

void setup() {
  Serial.begin(9600);
  nRF.begin();
  nRF.openWritingPipe(addresses[0]);
  nRF.openReadingPipe(1, addresses[1]);
  nRF.setPALevel(RF24_PA_LOW);
  nRF.startListening();
  
  for (int i = 0; i < countSensors; i++)
  {
  sensors[i].id=1+i;
  sensors[i].pinSensor = 2+i;
  sensors[i].pinLed = 9+i;
    pinMode(sensors[i].pinSensor, INPUT);
    digitalWrite(sensors[i].pinLed, LOW);
    pinMode(sensors[i].pinLed, OUTPUT);
  }
  testLight();
}

void loop() {
  for (int i = 1; i < countSensors; i++)
  {
    if (!digitalRead(sensors[i].pinSensor) && ! sensors[i].switchOn)
    {
      switchSensor(i);
    }
  }

  int code;
  if ( nRF.available()) {
    while (nRF.available()) {
      nRF.read( &code, sizeof(code) );
    }
    resetSensor(code -1);
  }

}

void resetSensor(int i)
{
  sensors[i].pinLed = LOW;
  sensors[i].switchOn = false;
}

void switchSensor(int i)
{
  sendCode(sensors[i].id);
  sensors[i].switchOn = HIGH;
  digitalWrite(sensors[i].pinLed, HIGH);
}

void sendCode(int code)
{
  nRF.stopListening();
  nRF.write( &code, sizeof(code) );
  nRF.startListening();
}

/*test method*/
void testLight()
{
  //rosvit
  switchLight(HIGH);
  //zhasni
  switchLight(LOW);
  //rosvit
  switchLight(HIGH);
  //zhasni
  switchLight(LOW);
}
void switchLight(boolean value )
{
  for (int i = 0; i < countSensors; i++)
  {
    digitalWrite(sensors[i].pinLed, value);
  }
  delay(500);
}